// Generated by CoffeeScript 1.3.1
/*
Queries are build using the factory functions in this module
The query manager classes use the following inheritance hierarchy:

  * BaseQuery

    * InsertQuery

    * SUDQuery

      * SelectQuery

      * UpdateQuery

      * DeleteQuery

The query factory functions defined here are re-exported by the main gesundheit
module. They each accept an optional visitor callback as a final parameter that
will be passed to the :meth:`~queries/base::BaseQuery.visit` method of the
newly created query.
*/

var DELETE, DeleteQuery, INSERT, InsertQuery, SELECT, SelectQuery, Tuple, UPDATE, UpdateQuery, maybeVisit;

Tuple = require('./../nodes').Tuple;

InsertQuery = require('./insert');

SelectQuery = require('./select');

UpdateQuery = require('./update');

DeleteQuery = require('./delete');

INSERT = function(table, fieldsOrRow) {
  /*
    Create a new :class:`queries/insert::InsertQuery` that will add rows to
    ``table``.
  
    :param fieldsOrRow: Either an array of column names that will be inserted, or a
      plain object representing a row of data to insert, in which case the keys
      of the object will define the columns that are inserted.
    :param visitor: (Optional) a function that will be called with it's context
      set to the newly constructed query object.
  */

  var fields, iq, row;
  if (fieldsOrRow && typeof fieldsOrRow === 'object') {
    if (Array.isArray(fieldsOrRow)) {
      fields = fieldsOrRow;
    } else {
      row = fieldsOrRow;
      fields = Object.keys(row);
    }
  }
  if (!(fields != null ? fields.length : void 0)) {
    throw new Error("Column list is required when constructing an INSERT");
  }
  iq = new InsertQuery(this, {
    table: table
  });
  iq.q.columns = iq.q.nodes[1] = new Tuple(fields);
  if (row) {
    iq.addRow(row);
  }
  return iq;
};

SELECT = function(table, fields) {
  /*
    Create a new :class:`queries/select::SelectQuery` selecting from ``table``.
  
    :param table: Table name to select rows from.
    :param fields: (Optional) Fields to project from ``table``. If this is not
      given, all fields (``*``) will be projected until
      :meth:`queries/select::SelectQuery.fields` is called.
    :param visitor: (Optional) a function that will be called with it's context
      set to the newly constructed query object.
  */

  var query;
  query = new SelectQuery(this, {
    table: table
  });
  if (fields != null) {
    query.fields.apply(query, fields);
  }
  return query;
};

UPDATE = function(table) {
  /*
    Create a new :class:`queries/update::UpdateQuery` that will update ``table``.
  
    :param visitor: (Optional) a function that will be called with it's context
      set to the newly constructed query object.
  */
  return new UpdateQuery(this, {
    table: table
  });
};

DELETE = function(table) {
  /*
    Create a new :class:`queries/delete::DeleteQuery` that will delete rows from
    ``table``.
  
    :param visitor: (Optional) a function that will be called with it's context
      set to the newly constructed query object.
  */
  return new DeleteQuery(this, {
    table: table
  });
};

exports.mixinFactoryMethods = function(invocant, getEngine) {
  /*
    Add wrappers methods for each of the query factory functions to ``invocant``
    using lower, UPPER, and Camel cases. The new methods will retrieve an engine
    using ``getEngine`` and then create the query bound to that engine.
  
    If ``getEngine`` is not given, queries will be bound to ``invocant`` instead.
  */

  var factory, type, _ref, _results;
  if (getEngine == null) {
    getEngine = function() {
      return invocant;
    };
  }
  _ref = {
    INSERT: INSERT,
    SELECT: SELECT,
    UPDATE: UPDATE,
    DELETE: DELETE
  };
  _results = [];
  for (type in _ref) {
    factory = _ref[type];
    _results.push((function(type, factory) {
      var wrapper;
      factory = maybeVisit(factory);
      wrapper = function() {
        return factory.apply(getEngine(), arguments);
      };
      invocant[type] = wrapper;
      invocant[type.toLowerCase()] = wrapper;
      return invocant[type[0] + type.toLowerCase().substring(1)] = wrapper;
    })(type, factory));
  }
  return _results;
};

maybeVisit = function(func) {
  return function() {
    var a, cb;
    a = [].slice.call(arguments);
    if (typeof a[a.length - 1] === 'function') {
      cb = a.pop();
      return func.apply(this, a).visit(cb);
    } else {
      return func.apply(this, a);
    }
  };
};
